package phonix;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.Set;
import java.util.Collections;

public class DatabaseConnector {
	private Connection connection;
	private  ResultSet result;
	private  ResultSet result2;
	private PreparedStatement pstat;
	private PreparedStatement pstat2;
	private static final String DBASE = System.getProperty("user.home")
			+ File.separatorChar + "PhonixJ" + File.separatorChar + "phonixj.db";
	private static final String CONFIG = System.getProperty("user.home")
			+ File.separatorChar + "PhonixJ" + File.separatorChar + "config";
	private static final String PHonixJ=System.getProperty("user.home")+File.separatorChar+"PhonixJ";
	
public DatabaseConnector(){
	try {
		File f;
		if(!(f=new File(PHonixJ)).exists()){
			f.mkdir();
		}
		Class.forName("org.sqlite.JDBC");
		File g=new File(CONFIG);
		String defa=DBASE;
		if(g.exists()){//The database isn't set to default.
			String m=null;
			BufferedReader br=new BufferedReader(new FileReader(g));
			while((m=br.readLine())!=null)defa=m;
			br.close();
		}
		connection = DriverManager.getConnection("jdbc:sqlite:"+defa);
		
	} catch (ClassNotFoundException e) {
		e.printStackTrace();
	} catch (SQLException e) {
		e.printStackTrace();
	} catch (FileNotFoundException e) {
		e.printStackTrace();
	} catch (IOException e) {
		e.printStackTrace();
	}
	
	try {
		pstat=connection.prepareStatement("SELECT * FROM person");
		result=pstat.executeQuery();
	} catch (SQLException e) {		
		createOne();
	}
}
public String insertPerson(Person person){
	String s=null;
	int key=0;
	//Check whether it is an existing place
	try {		
		s="select count(name) as countx from place where name=?";
		pstat=connection.prepareStatement(s);
		pstat.setString(1, person.getPlace());
		result=pstat.executeQuery();
		while(result.next()){
			key=result.getInt("countx");
		}
		//If doesn't exist insert one
		if(key==0){
			s="INSERT INTO place(name,zip) VALUES(?,?)";
			pstat=connection.prepareStatement(s);
			pstat.setString(1, person.getPlace());
			pstat.setString(2, person.getZIP());
			pstat.execute();
		}
		//Obtain the id and zip code for the place
		s="SELECT id,zip from place where name=?";
		pstat=connection.prepareStatement(s);
		pstat.setString(1, person.getPlace());
		result=pstat.executeQuery();		
		while(result.next()){
			key=result.getInt("id");
		}
		//If there are no zipcodes set yet set the actual input
		if(!person.getZIP().trim().equals("-")){			
			s="UPDATE place SET zip=? where name=?";
			pstat=connection.prepareStatement(s);
			pstat.setString(1, person.getZIP());
			pstat.setString(2, person.getPlace());
			pstat.execute();
		}
		//Insert the person into the person table
		s="INSERT INTO person(name,address,website,place_id,country) VALUES(?,?,?,?,?)";
		pstat=connection.prepareStatement(s);
		pstat.setString(1, person.getName());
		pstat.setString(2, person.getAddress());
		pstat.setString(3, person.getWebsite());
		pstat.setInt(4, key);
		pstat.setString(5, person.getCountry());
		pstat.execute();
		
		//get the id of the newly added person
		pstat=connection.prepareStatement("SELECT max(id) from person");
		result=pstat.executeQuery();		
		while(result.next()){
			key=result.getInt("max(id)");
		}
		//Insert the phone numbers
		for(String m:person.getNumber()){
		s="INSERT INTO phone(person_id,phone) VALUES(?,?)";
		pstat=connection.prepareStatement(s);
		pstat.setInt(1, key);
		pstat.setString(2, m);
		pstat.execute();
			}
		//Insert the email addresses
		for(String m:person.getEmail()){
			s="INSERT INTO email(person_id,email) VALUES(?,?)";
			pstat=connection.prepareStatement(s);
			pstat.setInt(1, key);
			pstat.setString(2, m);
			pstat.execute();
			}
		//Get the number of records
		pstat=connection.prepareStatement("SELECT COUNT(name) FROM person");
		result=pstat.executeQuery();
	} catch (SQLException e) {
		e.printStackTrace();
	}
	String p=null;
		try {
		p = ""+result.getInt("COUNT(name)");
		} catch (SQLException e) {
			e.printStackTrace();
		}	
	return p;
}

public void closeConnection(){
	if(connection!=null){
		try {
			connection.close();
		} catch (SQLException e) {
			e.printStackTrace();
		}
	}
}
// Find persons with names containing string s
public ArrayList<Person> findPerson(String s){
	Person person=null;
	String n=null;
	ArrayList<Person> personas=new ArrayList<>();	
	ArrayList<String> telefonok=new ArrayList<>();
	ArrayList<String> mailek=new ArrayList<>();
	String name=null;
	String website=null;
	String address=null;
	String placename=null;
	String zipcode=null;
	String country=null;
	int ID=0;
	int place_id=0;
			n="SELECT * FROM person WHERE name like ? order by name";				
				try {
					pstat=connection.prepareStatement(n);
					pstat.setString(1, "%"+s+"%");
					result=pstat.executeQuery();	
					
					while(result.next()){
						name=result.getString("name");
						ID=result.getInt("id");
						website=result.getString("website");
						address=result.getString("address");
						place_id=result.getInt("place_id");
						country=result.getString("country");
						//
						n="SELECT name,zip FROM place WHERE id=? LIMIT 1;";
						pstat2=connection.prepareStatement(n);
						pstat2.setInt(1, place_id);				
						result2=pstat2.executeQuery();
						while(result2.next()){
							placename=result2.getString("name");
							zipcode=result2.getString("zip");
						}
						
						n="SELECT phone FROM phone WHERE person_id=?";
						pstat2=connection.prepareStatement(n);
						pstat2.setInt(1, ID);				
						result2=pstat2.executeQuery();
						while(result2.next()){
							telefonok.add(result2.getString("phone"));				
						}
						
						n="SELECT email FROM email WHERE person_id=?";
						pstat2=connection.prepareStatement(n);
						pstat2.setInt(1, ID);				
						result2=pstat2.executeQuery();
						while(result2.next()){
							mailek.add(result2.getString("email"));				
						}							
						person = new Person.Builder().name(name)
								.number(telefonok).address(address)
								.place(placename).ZIP(zipcode)
								.website(website).email(mailek).ID(ID).country(country)
								.build();
						personas.add(person);
						telefonok.clear();
						mailek.clear();
					}
				} catch (SQLException e) {
					e.printStackTrace();
				}
				
				
			
			
		
	return personas;
}

public ArrayList<Two> getContacts(){
	ArrayList<Two> conti=new ArrayList<>();
	try {
		String s="SELECT id,name FROM person ORDER by name ASC";
		pstat=connection.prepareStatement(s);
		result=pstat.executeQuery();
		while(result.next()){
			conti.add(new Two(result.getInt("id"),result.getString("name")));
		}
	} catch (SQLException e) {
		e.printStackTrace();
	}
	return conti;
}

public ArrayList<Person> findInAll(String s){
	Person person=null;
	String[] query={"select id as ID from person where country like ?","select id as ID from person where name like ?","select id as ID from person where address like ?",
			"select id as ID from person where website like ?","select distinct person_id as ID from phone where phone like ?",
			"select person_id as ID from email where email like ?",
			"select distinct person.id as ID from person join place on person.place_id=place.id where (place.zip like ?)",
			"select distinct person.id as ID from person join place on person.place_id=place.id where (place.name like ?)"};
	Set<Integer> ids=new HashSet<>();
	ArrayList<Person> personas=new ArrayList<>();
	String k=null;
	try {
		for(String m:query){
			pstat=connection.prepareStatement(m);
			pstat.setString(1,"%"+ s+"%");
			result=pstat.executeQuery();
		while(result.next()){
			ids.add(result.getInt("ID"));
		}
		}
		int place_id=0;
		String name=null;
		String website=null;
		String address=null;
		String placename=null;
		String zipcode=null;
		String country=null;
		ArrayList<String> telefonok=null;
		ArrayList<String> mailek=null;
		for(Integer i:ids){
			telefonok=new ArrayList<>();
			mailek=new ArrayList<>();
			k="select name,address,website,country,place_id from person where id=?";
			pstat=connection.prepareStatement(k);
			pstat.setInt(1,i.intValue());
			result=pstat.executeQuery();
			while(result.next()){
				name=result.getString("name");
				website=result.getString("website");
				address=result.getString("address");	
				place_id=result.getInt("place_id");
				country=result.getString("country");
			}
			pstat=connection.prepareStatement("select name,zip from place where id=?");
			pstat.setInt(1,place_id);
			result=pstat.executeQuery();
			while(result.next()){
				placename=result.getString("name");
				zipcode=result.getString("zip");
			}
			
			pstat=connection.prepareStatement("select phone from phone where person_id=?");
			pstat.setInt(1,i.intValue());
			result=pstat.executeQuery();
			while(result.next()){
				telefonok.add(result.getString("phone"));
			}
			pstat=connection.prepareStatement("select email from email where person_id=?");
			pstat.setInt(1,i.intValue());
			result=pstat.executeQuery();
			while(result.next()){
				mailek.add(result.getString("email"));				
			}
			
			person = new Person.Builder().name(name)
					.number(telefonok).address(address)
					.place(placename).ZIP(zipcode)
					.website(website).email(mailek).ID(i).country(country)
					.build();
			personas.add(person);	
		}
		
			
		
	} catch (SQLException e) {
		e.printStackTrace();
	}
	Collections.sort(personas);
	return personas;
} 

public void delete(Person p){
	
	try {
		pstat=connection.prepareStatement("delete from person where id=?");
		pstat.setInt(1,p.getID());
		pstat.execute();
		
		pstat=connection.prepareStatement("delete from phone where person_id=?");
		pstat.setInt(1,p.getID());
		pstat.execute();
		
		pstat=connection.prepareStatement("delete from email where person_id=?");
		pstat.setInt(1,p.getID());
		pstat.execute();
		
		pstat=connection.prepareStatement("delete from groups where person_id=?");
		pstat.setInt(1,p.getID());
		pstat.execute();
	} catch (SQLException e) {
		e.printStackTrace();
	}
}

public String cityFinder(String s){
	 String n="";
	 
	try {
		
		pstat=connection.prepareStatement("SELECT DISTINCT name FROM place where name like ? order by length(name) LIMIT 1;");
		pstat.setString(1, s+"%");	
		result=pstat.executeQuery();
		while(result.next()){
			n=result.getString("name");
		}
	} catch (SQLException e) {
		e.printStackTrace();
	}
	return n;
}
public ArrayList<String> getAllCities(){
	ArrayList<String> varos=new ArrayList<>();
	try {
		pstat=connection.prepareStatement("SELECT DISTINCT name FROM place;");
		result=pstat.executeQuery();
		while(result.next()){
			varos.add(result.getString("name"));
		}
	} catch (SQLException e) {
		e.printStackTrace();
	}
	return varos;
}

public String ZipFinder(String s){
	String n="";
	try {
		pstat=connection.prepareStatement("SELECT zip FROM place WHERE name=? ORDER BY zip ASC;");
		pstat.setString(1,s);
		result=pstat.executeQuery();
		while(result.next()){
			if(!result.getString("zip").equals("-"))
			n=result.getString("zip");break;
		}
	} catch (SQLException e) {
		e.printStackTrace();
	}
	return n;
}
public void update(Person p,Person p2){
	int mid=0;
	try {		
		pstat=connection.prepareStatement("update person set name=?,address=?,website=?,country=? where id=? AND name=? AND address=? AND website=? AND country=?");
		pstat.setString(1,p2.getName());
		pstat.setString(2,p2.getAddress());
		pstat.setString(3,p2.getWebsite());
		pstat.setString(4,p2.getCountry());
		pstat.setInt(5,p.getID());
		pstat.setString(6,p.getName());
		pstat.setString(7,p.getAddress());
		pstat.setString(8,p.getWebsite());
		pstat.setString(9,p.getCountry());
		pstat.execute();
		
		
		pstat=connection.prepareStatement("delete from email where person_id=?");
		pstat.setInt(1,p.getID());
		pstat.execute();
		if(p2.getEmail().size()>0){
			for(String m:p2.getEmail()){
				pstat=connection.prepareStatement("insert into email(person_id,email) values(?,?);");				
				pstat.setInt(1,p.getID());
				pstat.setString(2,m);
				pstat.execute();
			}
		}
		
		pstat=connection.prepareStatement("delete from phone where person_id=?");
		pstat.setInt(1,p.getID());
		pstat.execute();
		if(p2.getNumber().size()>0){
			for(String m:p2.getNumber()){
				pstat=connection.prepareStatement("insert into phone(person_id,phone) values(?,?);");
				pstat.setInt(1,p.getID());
				pstat.setString(2,m);
				pstat.execute();
			}
		}
		
		//Find the value of the place and zip
		pstat=connection.prepareStatement("select count(name) from place where name=?");
		pstat.setString(1,p2.getPlace());
		result=pstat.executeQuery();
		//The old place_id
		while(result.next()){
			mid=result.getInt("count(name)");
		}
		if(mid==0){// new place
			pstat=connection.prepareStatement("insert into place(name,zip) values(?,?)");
			pstat.setString(1,p2.getPlace());
			pstat.setString(2,p2.getZIP());
			pstat.execute();			
		}
		pstat=connection.prepareStatement("select id from place where name=?");
		pstat.setString(1,p2.getPlace());
		result=pstat.executeQuery();
		//get the new id
		
		while(result.next()){
			mid=result.getInt("id");// mid is the new id
			}
		pstat=connection.prepareStatement("update person set place_id=? where id=?");
		pstat.setInt(1,mid);
		pstat.setInt(2,p.getID());
		pstat.execute();
	} catch (SQLException e) {
		e.printStackTrace();
	}
}


public ArrayList<Person> getAll(){
	Person person=null;
	ArrayList<Person> personas=new ArrayList<>();
	ArrayList<Integer> indexes=new ArrayList<>();
	try {
		pstat=connection.prepareStatement("select id from person order by name;");
		result=pstat.executeQuery();
		while(result.next()){
			indexes.add(result.getInt("id"));
		}
		for(Integer i:indexes){
			person=findID(i.intValue());
			personas.add(person);
		}
	} catch (SQLException e) {
		e.printStackTrace();
	}
	return personas;
}






public boolean isValid(String db){
	boolean t=true;
	Connection alter = null;
	try {
		alter = DriverManager.getConnection("jdbc:sqlite:"+db);
		pstat=connection.prepareStatement("SELECT name,address,website FROM person LIMIT 1;");
		result=pstat.executeQuery();
	} catch (SQLException e) {
		t=false;
	}
	try {
		alter.close();
	} catch (SQLException e) {
		e.printStackTrace();
	}
	
	return t;
}
public boolean changeDB(String db){
	boolean t=true;
	this.closeConnection();	
	try {
		connection = DriverManager.getConnection("jdbc:sqlite:"+db);
		createOne();
	} catch (SQLException e) {
		t=false;
	}
	return t;
}
private void createOne(){
	try {
		pstat=connection.prepareStatement("CREATE TABLE place(id INTEGER PRIMARY KEY ,name TEXT NOT NULL,zip TEXT);");
		pstat.execute();
		pstat=connection.prepareStatement("CREATE TABLE person(id INTEGER PRIMARY KEY ,name TEXT NOT NULL,address TEXT,website TEXT,place_id INTEGER,country TEXT,notes TEXT,FOREIGN KEY(place_id) REFERENCES place(id) ON UPDATE CASCADE ON DELETE CASCADE);");
		pstat.execute();
		pstat=connection.prepareStatement("CREATE TABLE email(person_id INTEGER ,email TEXT NOT NULL,FOREIGN KEY(person_id) REFERENCES person(id) ON UPDATE CASCADE ON DELETE CASCADE);");
		pstat.execute();
		pstat=connection.prepareStatement("CREATE TABLE phone(person_id INTEGER ,phone TEXT NOT NULL,FOREIGN KEY(person_id) REFERENCES person(id) ON UPDATE CASCADE ON DELETE CASCADE);");
		pstat.execute();
		pstat=connection.prepareStatement("CREATE TABLE categories(id INTEGER PRIMARY KEY ,name TEXT NOT NULL)");
		pstat.execute();
		pstat=connection.prepareStatement("CREATE TABLE groups(person_id INTEGER,cat_id INTEGER,FOREIGN KEY(person_id) REFERENCES person(id) ON UPDATE CASCADE ON DELETE CASCADE,FOREIGN KEY(cat_id) REFERENCES categories(id) ON UPDATE CASCADE ON DELETE CASCADE)");
		pstat.execute();
	} catch (SQLException e) {
		
	}
	
}
public Person findID(int id){
	Person person=null;
	try {		
		pstat=connection.prepareStatement("SELECT * FROM person WHERE id=?");
		pstat.setInt(1, id);
		result=pstat.executeQuery();
		String name=null;
		String website=null;
		String address=null;
		String placename=null;
		String zipcode=null;
		String country=null;
		ArrayList<String> telefonok=new ArrayList<>();
		ArrayList<String> mailek=new ArrayList<>();
		int ID=0;
		int place_id=0;
		while(result.next()){
			name=result.getString("name");
			ID=result.getInt("id");
			website=result.getString("website");
			address=result.getString("address");
			place_id=result.getInt("place_id");
			country=result.getString("country");
			pstat2=connection.prepareStatement("SELECT name,zip FROM place WHERE id=? LIMIT 1;");
			pstat2.setInt(1, place_id);
			result2=pstat2.executeQuery();
			while(result2.next()){
				placename=result2.getString("name");
				zipcode=result2.getString("zip");
			}
			pstat2=connection.prepareStatement("SELECT phone FROM phone WHERE person_id=?");
			pstat2.setInt(1, ID);
			result2=pstat2.executeQuery();
			while(result2.next()){
				telefonok.add(result2.getString("phone"));				
			}
			pstat2=connection.prepareStatement("SELECT email FROM email WHERE person_id=?");
			pstat2.setInt(1, ID);
			result2=pstat2.executeQuery();
			while(result2.next()){
				mailek.add(result2.getString("email"));				
			}		
			
			person = new Person.Builder().name(name)
					.number(telefonok).address(address)
					.place(placename).ZIP(zipcode)
					.website(website).email(mailek).ID(ID).country(country)
					.build();
		}
	} catch (SQLException e) {
		e.printStackTrace();
	}
	return person;
	}

public ArrayList<Person> findInColumn(String s,String column){
	Person person=null;
	Set<Integer> ids=new HashSet<>();
	ArrayList<Person> personas=new ArrayList<>();
	String m=null;
	switch(column){
	case "name":m="select id as ID from person where name like ?";break;
	case "address": m="select id as ID from person where address like ?";break;
	case "website": m="select id as ID from person where website like ?";break;
	case "phone": m="select distinct person_id as ID from phone where phone like ?";break;
	case "email": m="select person_id as ID from email where email like ?";break;
	case "zip": m="select distinct person.id as ID from person join place on person.place_id=place.id where (place.zip like ?)";break;
	case "place": m="select distinct person.id as ID from person join place on person.place_id=place.id where (place.name like ?)";break;
	case "country":m="select id as ID from person where country like ?";break;
	}
	try {
		pstat=connection.prepareStatement(m);
		pstat.setString(1,"%"+s+"%");
		result=pstat.executeQuery();
		while(result.next()){
			ids.add(result.getInt("ID"));
		}
		int place_id=0;
		String name=null;
		String website=null;
		String address=null;
		String placename=null;
		String zipcode=null;
		String country=null;
		ArrayList<String> telefonok=null;
		ArrayList<String> mailek=null;
		for(Integer i:ids){
			telefonok=new ArrayList<>();
			mailek=new ArrayList<>();
			pstat=connection.prepareStatement("select name,address,website,place_id,country from person where id=?");
			pstat.setInt(1,i.intValue());
			result=pstat.executeQuery();
			while(result.next()){
				name=result.getString("name");
				website=result.getString("website");
				address=result.getString("address");	
				place_id=result.getInt("place_id");
				country=result.getString("country");
			}
			pstat=connection.prepareStatement("select name,zip from place where id=?");
			pstat.setInt(1,place_id);
			result=pstat.executeQuery();
			while(result.next()){
				placename=result.getString("name");
				zipcode=result.getString("zip");
			}
			pstat=connection.prepareStatement("select phone from phone where person_id=?");
			pstat.setInt(1,i.intValue());
			result=pstat.executeQuery();
			while(result.next()){
				telefonok.add(result.getString("phone"));
			}
			pstat=connection.prepareStatement("select email from email where person_id=?");
			pstat.setInt(1,i.intValue());
			result=pstat.executeQuery();
			while(result.next()){
				mailek.add(result.getString("email"));
			}
			person = new Person.Builder().name(name)
					.number(telefonok).address(address)
					.place(placename).ZIP(zipcode).country(country)
					.website(website).email(mailek).ID(i)
					.build();
			personas.add(person);	
		}
		
			
		
	} catch (SQLException e) {
		e.printStackTrace();
	}
	Collections.sort(personas);
	return personas;
} 


public int findOrInsert(String p,String z){
	int id=0;
	try {
		pstat=connection.prepareStatement("select count(name) from place where name=?");
		pstat.setString(1,p);
		result=pstat.executeQuery();
		int x=result.getInt("count(name)");
		if(x==0){
			pstat=connection.prepareStatement("insert into place(name,zip) values(?,?)");
			pstat.setString(1,p);
			pstat.setString(2,z);
			pstat.execute();
			pstat=connection.prepareStatement("select id from place where name=?");
			pstat.setString(1,p);
			result=pstat.executeQuery();
			id=result.getInt("id");
		}else{
			pstat=connection.prepareStatement("select id from place where name=?");
			pstat.setString(1,p);
			result=pstat.executeQuery();
			id=result.getInt("id");
		}
	} catch (SQLException e) {
		e.printStackTrace();
	}
	return id;
}

public ArrayList<Two> getCategories(){
	ArrayList<Two> kateg=new ArrayList<>();
	
	try {
		pstat=connection.prepareStatement("select * from categories order by name");
		result=pstat.executeQuery();
		while(result.next()){
			kateg.add(new Two(result.getInt("id"),result.getString("name")));
		}
	} catch (SQLException e) {
		e.printStackTrace();
	}
	return kateg;
}

public ArrayList<String> getGroups(int id){
	ArrayList<String> groups=new ArrayList<>();
	try {
		pstat=connection.prepareStatement("select categories.name as NEV from groups join categories on groups.cat_id=categories.id where groups.person_id=? order by categories.name");
		pstat.setInt(1, id);
		result=pstat.executeQuery();
		while(result.next()){
			groups.add(result.getString("NEV"));
		}
	} catch (SQLException e) {
		e.printStackTrace();
	}
	return groups;
	
}

public void deleteCategory(String name){
	int q=0;
	try {
		pstat=connection.prepareStatement("select id from categories where name=?");
		pstat.setString(1, name);
		result=pstat.executeQuery();
		while(result.next()){
			q=result.getInt("id");
		}
		pstat=connection.prepareStatement("delete from categories where name=?");
		pstat.setString(1, name);
		pstat.execute();
		
		pstat=connection.prepareStatement("delete from groups where cat_id=?");
		pstat.setInt(1, q);
		pstat.execute();
	} catch (SQLException e) {
		e.printStackTrace();
	}
}

public void createCategory(String name){
	try {
		pstat=connection.prepareStatement("insert into categories(name) values(?);");
		pstat.setString(1, name);
		pstat.execute();
	} catch (SQLException e) {
		e.printStackTrace();
	}
	
}

public void insertIntoGroup(int id,String name){
	int g=0;
	int db=0;
	try {
		pstat=connection.prepareStatement("select id from categories where name=?;");
		pstat.setString(1, name);
		result=pstat.executeQuery();
		while(result.next())
		g=result.getInt("id");
		
		pstat=connection.prepareStatement("select count(*) from groups where person_id=? and cat_id=?;");
		pstat.setInt(1, id);
		pstat.setInt(2, g);
		result=pstat.executeQuery();
		while(result.next()){
			db=result.getInt("count(*)");
		}
		if(db==0){
		pstat=connection.prepareStatement("insert into groups(person_id,cat_id) values(?,?);");
		pstat.setInt(1, id);
		pstat.setInt(2, g);
		pstat.execute();
		}
	} catch (SQLException e) {
		e.printStackTrace();
	}
	
}
public void removeFromGroup(int id,String name){
	int g=0;
	try {
		pstat=connection.prepareStatement("select id from categories where name=?;");
		pstat.setString(1, name);
		result=pstat.executeQuery();
		while(result.next())
		g=result.getInt("id");
		
		pstat=connection.prepareStatement("delete from groups where person_id=? and cat_id=?;");
		pstat.setInt(1, id);
		pstat.setInt(2, g);
		pstat.execute();
		
	} catch (SQLException e) {
		e.printStackTrace();
	}
	
	
}
	public ArrayList<Person> findInCategories(String s,String mode){
		Person person=null;
		Set<Integer> ids=new HashSet<>();
		ArrayList<Person> personas=new ArrayList<>();
		try {
			if(mode.equals("QUERY")){
			pstat=connection.prepareStatement("select distinct person.id as ID from person join groups on person.id=groups.person_id join categories on groups.cat_id=categories.id where person.name like ? order by person.name");
			pstat.setString(1,"%"+s+"%");
			}else{
				pstat=connection.prepareStatement("select distinct person.id as ID from person join groups on person.id=groups.person_id join categories on groups.cat_id=categories.id where categories.name=? order by person.name");
				pstat.setString(1,s);
			}
			result=pstat.executeQuery();
			while(result.next()){
				ids.add(result.getInt("ID"));
			}
			int place_id=0;
			String name=null;
			String website=null;
			String address=null;
			String placename=null;
			String zipcode=null;
			String country=null;
			ArrayList<String> telefonok=null;
			ArrayList<String> mailek=null;
			for(Integer i:ids){
				telefonok=new ArrayList<>();
				mailek=new ArrayList<>();
				pstat=connection.prepareStatement("select name,address,website,place_id,country from person where id=?");
				pstat.setInt(1,i.intValue());
				result=pstat.executeQuery();
				while(result.next()){
					name=result.getString("name");
					website=result.getString("website");
					address=result.getString("address");	
					place_id=result.getInt("place_id");
					country=result.getString("country");
				}
				pstat=connection.prepareStatement("select name,zip from place where id=?");
				pstat.setInt(1,place_id);
				result=pstat.executeQuery();
				while(result.next()){
					placename=result.getString("name");
					zipcode=result.getString("zip");
				}
				pstat=connection.prepareStatement("select phone from phone where person_id=?");
				pstat.setInt(1,i.intValue());
				result=pstat.executeQuery();
				while(result.next()){
					telefonok.add(result.getString("phone"));
				}
				pstat=connection.prepareStatement("select email from email where person_id=?");
				pstat.setInt(1,i.intValue());
				result=pstat.executeQuery();
				while(result.next()){
					mailek.add(result.getString("email"));
				}
				person = new Person.Builder().name(name)
						.number(telefonok).address(address)
						.place(placename).ZIP(zipcode).country(country)
						.website(website).email(mailek).ID(i)
						.build();
				personas.add(person);	
			}
			
				
			
		} catch (SQLException e) {
			e.printStackTrace();
		}
		return personas;
	}
	
	//Guess the home country for user
	/**
	 * @return the most possible country of residence for the end-user
	 * @parameters none
	 */
	public String homeGuess(){
		String coun=null;
		try {
			pstat=connection.prepareStatement("SELECT orszagok.country as HIGH, orszagok.osszeg FROM (SELECT country, count(country) as osszeg FROM person GROUP BY country ORDER BY osszeg) as orszagok where orszagok.country<>'-' ORDER BY orszagok.osszeg DESC LIMIT 1");
			result=pstat.executeQuery();
			
			while(result.next()){
				coun=result.getString("HIGH");
			}
		} catch (SQLException e) {
			e.printStackTrace();
		}
		if (coun==null)
			coun="-";
		return coun;
	}
	public String countryOfTown(String s){
		String t=null;
		try {
			pstat=connection.prepareStatement("SELECT person.country as orszag FROM person join place on person.place_id=place.id WHERE place.name=? AND person.country<>'-' LIMIT 1;");
			pstat.setString(1,s);
			result=pstat.executeQuery();
			
			while(result.next()){
				t=result.getString("orszag");
			}
		} catch (SQLException e) {
			e.printStackTrace();
		}
		
		return t;
	}
	
	public void fix(){
		try {
			pstat=connection.prepareStatement("alter table person add notes TEXT default '-'");
			pstat.execute();
			for(Country c:PUtils.getCountries()){
				pstat=connection.prepareStatement("update person set country=? where country=?");
				pstat.setString(1, c.getCode());
				pstat.setString(2, c.getName());
				pstat.execute();
			}
			
			
		} catch (SQLException e) {
			//e.printStackTrace();
		}
	}
	public ArrayList<Country> getMails(String category){
		ArrayList<Country> mails=new ArrayList<>();
		try {
			if(category.equals("*.*")){
				pstat=connection.prepareStatement("select person.name as NAME,email.email AS EMAIL from " +
						"person join email on person.id=email.person_id order by name");
			}else{
			pstat=connection.prepareStatement("select person.name as NAME,email.email AS EMAIL from " +
					"person join email on person.id=email.person_id " +
					"join groups on groups.person_id=person.id " +
					"join categories on categories.id=groups.cat_id " +
					"where categories.name=? order by name");
			pstat.setString(1, category);
			}
			result=pstat.executeQuery();
			while(result.next()){				
				mails.add(new Country(result.getString("EMAIL"),result.getString("NAME")));				
			}
		} catch (SQLException e) {
			e.printStackTrace();
		}
		return mails;
	}
	public String getNotes(int iD) {
		String t=null;
		try {
			pstat=connection.prepareStatement("SELECT notes from person where id=?;");
			pstat.setInt(1,iD);
			result=pstat.executeQuery();
			
			while(result.next()){
				t=result.getString("notes");
			}
		} catch (SQLException e) {
			e.printStackTrace();
		}
		
		return t;
	}
	public void saveNotes(String text,int ID) {
		try {
			pstat=connection.prepareStatement("UPDATE person set notes=? where id=?;");
			pstat.setString(1,text);
			pstat.setInt(2,ID);
			pstat.execute();
		} catch (SQLException e) {
			e.printStackTrace();
		}
		
	}

}



