package phonix;


import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;

import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.printing.Printer;

class WrappingPrinter {
	  private Printer printer; 
	  private String fileName;
	  private String contents;
	  private GC gc;
	  private int xPos, yPos;
	  private Rectangle bounds;// The boundaries for the print
	  private StringBuffer buf;
	  private int lineHeight;
	  WrappingPrinter(Printer printer,String filename) {
	    this.printer = printer;
	    this.fileName = filename;
	    StringBuffer sb=new StringBuffer();
	    String s=null;
	    try {
			BufferedReader br=new BufferedReader(new FileReader(new File(this.fileName)));
			while((s=br.readLine())!=null){
				sb.append(s+"\n");
			}
			br.close();
			contents=sb.toString();
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}

	  }

	  /**
	   * Prints the file
	   */
	  void print() {
	    // Start the print job
	    if (printer.startJob(fileName)) {
	      // Determine print area, with margins
	      bounds = computePrintArea(printer);
	      xPos = bounds.x;
	      yPos = bounds.y;

	      // Create the GC
	      gc = new GC(printer);

	      // Determine line height
	      lineHeight = gc.getFontMetrics().getHeight();

	      // Determine tab width--use three spaces for tabs
	      int tabWidth = gc.stringExtent(" ").x;

	      // Print the text
	      printer.startPage();
	      buf = new StringBuffer();
	      char c;
	      for (int i = 0, n = contents.length(); i < n; i++) {
	        // Get the next character
	        c = contents.charAt(i);

	        // Check for newline
	        if (c == '\n') {
	          printBuffer();
	          printNewline();
	        }
	        // Check for tab
	        else if (c == '\t') {
	          xPos += tabWidth;
	        } else {
	          buf.append(c);
	          // Check for space
	          if (Character.isWhitespace(c)) {
	            printBuffer();
	          }
	        }
	      }
	      printer.endPage();
	      printer.endJob();
	      gc.dispose();
	    }
	  }

	  /**
	   * Prints the contents of the buffer
	   */
	  void printBuffer() {
	    // Get the width of the rendered buffer
	    int width = gc.stringExtent(buf.toString()).x;

	    // Determine if it fits
	    if (xPos + width > bounds.x + bounds.width) {
	      // Doesn't fit--wrap
	      printNewline();
	    }

	    // Print the buffer
	    gc.drawString(buf.toString(), xPos, yPos, false);
	    xPos += width;
	    buf.setLength(0);
	  }

	  /**
	   * Prints a newline
	   */
	  void printNewline() {
	    // Reset x and y locations to next line
	    xPos = bounds.x;
	    yPos += lineHeight;

	    // Have we gone to the next page?
	    if (yPos > bounds.y + bounds.height) {
	      yPos = bounds.y;
	      printer.endPage();
	      printer.startPage();
	    }
	  }

	  /**
	   * Computes the print area, including margins
	   * 
	   * @param printer
	   *          the printer
	   * @return Rectangle
	   */
	  Rectangle computePrintArea(Printer printer) {
	    Rectangle rect = printer.getClientArea();
	    Rectangle trim = printer.computeTrim(0, 0, 0, 0);
	    Point dpi = printer.getDPI();
	    // Calculate the printable area, using 1 inch margins
	    int left = trim.x + dpi.x/4;
	    if (left < rect.x)
	      left = rect.x;
	    int right = (rect.width + trim.x + trim.width) - dpi.x/4;
	    if (right > rect.width)
	      right = rect.width;
	    int top = trim.y + dpi.y/4;
	    if (top < rect.y)
	      top = rect.y;
	    int bottom = (rect.height + trim.y + trim.height) - dpi.y/4;
	    if (bottom > rect.height)
	      bottom = rect.height;
	    return new Rectangle(left, top, right - left, bottom - top);
	  }
	}
